import argparse

class NotErrorArgumentParser(argparse.ArgumentParser):
    """
    if arg is not defined, the class will be silent.
    """

    def parse_args(self, args=None, namespace=None):
        args, argv = self.parse_known_args(args, namespace)
        return args



import re

PATTERN = re.compile("\d+\.\d+\.\d+")


def read_setup(file: str) -> str:
    with open(file, encoding='utf-8', mode='r') as f:
        return f.read()


def update_version(file: str, version: str):
    subbed = PATTERN.sub(version, read_setup(file))
    with open(file, encoding='utf-8', mode='w') as fl:
        fl.write(subbed)


def current_version(file: str):
    return PATTERN.findall(read_setup(file))



import base64
from typing import Union

from nezha.ustring import to_bytes, to_str


class Picture:

    @staticmethod
    def to_base64(s: Union[str, bytes, bytearray], is_file: bool = False) -> str:
        if is_file and isinstance(s, str):
            with open(s, mode='rb') as f:
                content = f.read()
        else:
            content = to_bytes(s)
        return to_str(base64.b64encode(content))



import re
import secrets
import uuid
from typing import Union, List, Tuple

PATTERN_CHINESE_NAME = re.compile(r'[^\u4e00-\u9fa5·]+')


def remove_invisible(s: str) -> str:
    """
    support remove \t \n \xa0 ...
    :param s:
    :return:
    """
    return ''.join(s.split())


def unique_id() -> str:
    return str(uuid.uuid1())


def to_bytes(s: Union[str, bytes]) -> bytes:
    if isinstance(s, (bytes, bytearray)):
        return s
    return s.encode('utf-8')


def to_str(b: Union[bytes, str]) -> str:
    if isinstance(b, str):
        return b
    return b.decode('utf-8')


def random_key(length: int) -> str:
    if not isinstance(length, int):
        raise Exception(f'nbytes {length} type {type(length)} not int')
    return secrets.token_hex(nbytes=length)[0:length]


def camelize2underline(camelize_words: str, separator: str = "_") -> str:
    u"""
    驼峰转下划线的方法；
    :param camelize_words: 需要转化的字符串
    :param separator: _
    :return: 字符串
    """
    sep = separator + r'\1'
    if not camelize_words:
        return ""
    if not isinstance(camelize_words, str):
        raise ValueError('camelize_words {} must be str'.format(camelize_words))
    pattern = re.compile(r'([A-Z]{1})')
    sub = re.sub(pattern, sep, camelize_words).lower()
    return sub


def is_chinese_name(value: str) -> bool:
    u"""
    校验姓名是否是由汉字或汉字和·组成（王大锤 or 买买提·古力娜扎·迪丽热巴）
    :param value: name type is str
    :return:bool  符合格式的name返回 True 否则返回False
    """
    if not isinstance(value, str):
        raise ValueError("value {} type must be string".format(value))
    return PATTERN_CHINESE_NAME.match(value) is not None


def clean_str(s: str, striped: Union[List[str], str, None] = None) -> str:
    """

    :param s:
    :param striped: need strip character.
    :return:
    """
    s = str(s)
    striped = striped or []
    striped = striped if isinstance(striped, list) else [striped, ]
    for chars in striped:
        s = s.strip(chars)
    return ''.join(s.split())


def replace_chinese_comma(s: str) -> str:
    chinese_comma = '，'
    english_comma = ','
    return s.replace(chinese_comma, english_comma)


def is_invalid_str(*s: str,
                   invalid_str: Tuple[str, ...] = ('', '\n', '\t'),
                   invalid_length: Tuple[int, ...] = (0, 1)
                   ) -> bool:
    rules = (
        lambda *x: set(i in invalid_str for i in x) != {False},
        # debug: i or '' is used for i is None.
        lambda *x: set(len(i or '') in invalid_length for i in x) != {False},
    )
    for hit_rule in rules:
        if hit_rule(*s):
            return True
    return False


if __name__ == '__main__':
    s = is_invalid_str('s2')
    print(s)



import oss2
from oss2 import Bucket

class OSS(object):
    """
    aliyun oss sdk
    """

    def __init__(self, access_key_id: str, access_key_secret: str, bucket_name: str, endpoint: str):
        self.bucket: Bucket = oss2.Bucket(oss2.Auth(access_key_id, access_key_secret), endpoint, bucket_name)
        self.get_file_expires: int = 3600 * 24

    def get(self, file_path: str) -> str:
        """
        从oss获取文件的url
        :param file_path: 文件的oss存储路径
        """
        return self.bucket.sign_url('GET', file_path, self.get_file_expires)

    def upload(self, file_path: str, file: str) -> bool:
        """
        上传文件到oss
        :param file_path: 文件的oss存储路径
        :param file: 上传文件
        :return: 上传状态
        """
        result = self.bucket.put_object(key=file_path, data=file)
        if result.status == 200:
            return True
        else:
            return False



from collections import OrderedDict
from typing import Union, Dict, Any,Mapping
from urllib import parse
from warnings import warn


def urljoin(base: str, url: str = '', params: Union[str, Mapping] = '') -> str:
    return '?'.join((parse.urljoin(base, url), parse.urlencode(params))).rstrip('?')


def urlencode(data: Dict[str, Any], safe: bool = True, sorted_by_ascii: bool = False) -> str:
    warn("deprecated", DeprecationWarning)
    if safe:
        return parse.urlencode(data)
    else:
        may_sorted = data.copy()
        if sorted_by_ascii:
            may_sorted = OrderedDict()
            for k in sorted(data):
                may_sorted[k] = data[k]
        return "&".join((f'{k}={v}' for k, v in may_sorted.items()))


if __name__ == '__main__':
    print(urljoin('http://ww.sx/s/'))



from functools import wraps
from typing import Callable, Iterable


def modify_positional_argument(index: int, val: str, action: str = 'replace', is_func_decorated: bool = True):
    """

    :param index: old positional argument index
    :param val:
    :param action:
        + if action == 'replace', old val -> val
        + if action == 'suffix', old val + val  -> val
        + if action == 'prefix', val + old val -> val
    :param is_func_decorated: if class method or instance method is decorated, self/cls should be passed.
    :return:

    example:

    @modify_positional_argument(0, 'xxxxx', action='prefix')
    def a(xx):
        print(xx)


    class A:

        @modify_positional_argument(0, 'xxxx', is_func_decorated=False)
        def a(self, x):
            print(x)
    """

    def action_factory(key: str) -> Callable:
        actions = {
            'replace': lambda old, new: new,
            'prefix': lambda old, new: new + old,
            'suffix': lambda old, new: old + new,
        }
        return actions[key]

    def decorate(func):
        @wraps(func)
        def wrap_method(self, *args, **kwargs):
            print(locals())
            old_val = args[index]
            print('old_val', old_val)
            new_args = list(args)
            new_args[index] = action_factory(action)(old_val, val)
            return func(self, *new_args, **kwargs)

        @wraps(func)
        def wrap_func(*args, **kwargs):
            print(locals())
            old_val = args[index]
            new_args = list(args)
            new_args[index] = action_factory(action)(old_val, val)
            return func(*new_args, **kwargs)

        return wrap_func if is_func_decorated else wrap_method

    return decorate


def loop(it: Iterable):
    while True:
        try:
            next(it)
        except StopIteration:
            break



import re
from typing import Mapping, Iterable, Dict

from nezha import utime
from pymongo import MongoClient

yesterday = lambda: re.compile(f'{utime.yesterday(precision="day")}.*')
today = lambda: re.compile(f'{utime.today(precision="day")}.*')


def insert_one(data: Mapping,
               collection: str, db: str, host: str, port: int, username: str, password: str,
               authSource: str) -> None:
    with MongoClient(host=host,
                     port=port,
                     username=username,
                     password=password,
                     authSource=authSource,
                     maxPoolSize=1) as MgClient:
        MgClient[db][collection].insert_one(data)


def find(condition: Mapping,
         collection: str, db: str, host: str, port: int, username: str, password: str,
         authSource: str) -> Iterable:
    with MongoClient(host=host,
                     port=port,
                     username=username,
                     password=password,
                     authSource=authSource,
                     maxPoolSize=1) as MgClient:
        return MgClient[db][collection].find(condition)


def find_one(condition: Mapping,
             collection: str, db: str, host: str, port: int, username: str, password: str,
             authSource: str) -> Dict:
    with MongoClient(host=host,
                     port=port,
                     username=username,
                     password=password,
                     authSource=authSource,
                     maxPoolSize=1) as MgClient:
        return MgClient[db][collection].find_one(condition)
